unit IAColorRichItem;

interface
Uses classes,graphics;

type
TIAitemclass = (iaicWord,iaicRange,iaicBrackets);
TIAWholeWord = (iawwYes,iawwNo);
TIAMatchCase = (iamcYes,iamcNo);

TIAColorRichItem  = class;
TIAColorItemsArray = array of TIAColorRichItem;
TIAColorRichItems = class(TPersistent)
private
  FonChange: TNotifyEvent;
  FItemsArray : TIAColorItemsArray;

  procedure ItemsGhanged(Sender: TObject);
protected
  procedure DefineProperties(Filer: TFiler); override;
  procedure ReadData(Stream: TStream);
  procedure WriteData(Stream: TStream);

  function GetItems(Index: Cardinal): TIAColorRichItem;         virtual;
  procedure SetItems(Index: Cardinal;Value: TIAColorRichItem);  virtual;
  procedure SetItemsCount(Value: Cardinal);                     virtual;
  function  GetItemsCount: Cardinal; virtual;

public
  constructor Create; virtual;
  destructor Destroy; override;
  procedure Assign(Source: TPersistent); override;

  procedure AddItem;
  procedure DeleteItem(Index : cardinal);
  procedure ClearListofItems;

  property onChange : TNotifyEvent read FonChange write FonChange;

  property ItemsCount : Cardinal read GetItemsCount write SetItemsCount;
  property Items[Index : Cardinal]  : TIAColorRichItem read GetItems write SetItems; default;
end;
TIAColorRichItem = class(TPersistent)
private
  FFontSize: integer;
  FFontColor: TColor;
  FFontName: TFontName;
  FFontStyle: TFontStyles;
  FonChange: TNotifyEvent;
  FRangeEnd: Byte;
  FRangeBegin: Byte;
  FLeftBracket: string;
  FRightBracket: string;
  FWordName: string;
  FPriority: integer;
  FItemClass: TIAitemclass;
  FWholeWord: TIAWholeWord;
  FMatchCase : TIAMatchCase;

protected

  procedure DefineProperties(Filer: TFiler); override;

  procedure SetFontColor(Value: TColor);           virtual;
  procedure SetFontName(Value: TFontName);         virtual;
  procedure SetFontSize(Value: integer);           virtual;
  procedure SetFontStyle(Value: TFontStyles);      virtual;
  procedure SetLeftBracket(Value: string);         virtual;
  procedure SetRangeBegin(Value: Byte);            virtual;
  procedure SetRangeEnd(Value: Byte);              virtual;
  procedure SetRightBracket(Value: string);        virtual;
  procedure SetWordName(Value: string);            virtual;
  procedure SetPriority(Value: integer);           virtual;
  procedure SetItemClass(Value: TIAitemclass);     virtual;
  procedure SetWholeWord(Value: TIAWholeWord);     virtual;
  procedure SetMatchCase(Value : TIAMatchCase);    virtual;

  procedure ReadData(Stream: TStream);
  procedure WriteData(Stream: TStream);
public
  constructor Create; virtual;
  destructor Destroy; override;
  procedure Assign(Source: TPersistent); override;

  property WordName     : string read FWordName     write SetWordName;
  property RangeBegin   : Byte   read FRangeBegin   write SetRangeBegin;
  property RangeEnd     : Byte   read FRangeEnd     write SetRangeEnd;
  property LeftBracket  : string read FLeftBracket  write SetLeftBracket;
  property RightBracket : string read FRightBracket write SetRightBracket;
  property Priority     : integer read FPriority write SetPriority;

  property FontName  : TFontName read FFontName write SetFontName;
  property FontSize  : integer read FFontSize write SetFontSize;
  property FontColor : TColor read FFontColor write SetFontColor;
  property FontStyle : TFontStyles read FFontStyle write SetFontStyle;

  property ItemClass : TIAitemclass read FItemClass write SetItemClass;
  property WholeWord : TIAWholeWord read FWholeWord write SetWholeWord;
  property MatchCase : TIAMatchCase read FMatchCase write SetMatchCase;

  property onChange : TNotifyEvent read FonChange write FonChange;
end;


implementation

{ TIAColorRichItem }

procedure TIAColorRichItem.Assign(Source: TPersistent);
begin
//  inherited Assign(Source);
  if Source is TIAColorRichItem then
    begin
      FFontSize:=(Source as TIAColorRichItem).FFontSize;
      FFontColor:=(Source as TIAColorRichItem).FFontColor;
      FFontName:=(Source as TIAColorRichItem).FFontName;
      FFontStyle:=(Source as TIAColorRichItem).FFontStyle;
      FRangeEnd:=(Source as TIAColorRichItem).FRangeEnd;
      FRangeBegin:=(Source as TIAColorRichItem).FRangeBegin;
      FLeftBracket:=(Source as TIAColorRichItem).FLeftBracket;
      FRightBracket:=(Source as TIAColorRichItem).FRightBracket;
      FWordName:=(Source as TIAColorRichItem).FWordName;
      FPriority:=(Source as TIAColorRichItem).FPriority;
      FWholeWord:=(Source as TIAColorRichItem).FWholeWord;
      FMatchCase:=(Source as TIAColorRichItem).FMatchCase;
      FItemClass:=(Source as TIAColorRichItem).FItemClass;

      if assigned(FonChange) then FOnChange(self);
    end;
end;

constructor TIAColorRichItem.Create;
begin
  inherited Create;
  FFontSize:=8;
  FFontColor:=clWindowText;
  FFontName:='Arial';
  FFontStyle:=[];
  FonChange:=nil;
  FRangeEnd:=0;
  FRangeBegin:=0;
  FLeftBracket:='';
  FRightBracket:='';
  FWordName:='';
  FPriority:=0;
  FWholeWord:=iawwNo;
  FMatchCase:=iamcNo;
end;

procedure TIAColorRichItem.DefineProperties(Filer: TFiler);
begin
  Filer.DefineBinaryProperty('Content',ReadData,WriteData,TRUE);
end;

destructor TIAColorRichItem.Destroy;
begin
  inherited Destroy;
end;

procedure TIAColorRichItem.ReadData(Stream: TStream);
  var i : cardinal;
      s : string;
begin
  Stream.ReadBuffer(i,4);FFontSize:=i;
  Stream.ReadBuffer(i,4);FFontColor:=TColor(i);
  Stream.ReadBuffer(i,4);
  if i>0 then
    begin
      setlength(s,i);
      Stream.ReadBuffer(s[1],i);
      FFontName:=s;
    end;
  Stream.ReadBuffer(i,4);if i<>0 then FFontStyle:=FFontStyle+[fsBold] else FFontStyle:=FFontStyle-[fsBold];
  Stream.ReadBuffer(i,4);if i<>0 then FFontStyle:=FFontStyle+[fsItalic] else FFontStyle:=FFontStyle-[fsItalic];
  Stream.ReadBuffer(i,4);if i<>0 then FFontStyle:=FFontStyle+[fsUnderline] else FFontStyle:=FFontStyle-[fsUnderline];
  Stream.ReadBuffer(i,4);if i<>0 then FFontStyle:=FFontStyle+[fsStrikeOut] else FFontStyle:=FFontStyle-[fsStrikeOut];
  Stream.ReadBuffer(i,4);FRangeEnd:=i;
  Stream.ReadBuffer(i,4);FRangeBegin:=i;
  Stream.ReadBuffer(i,4);
  if i>0 then
    begin
      setlength(s,i);
      Stream.ReadBuffer(s[1],i);
      FLeftBracket:=s;
    end;
  Stream.ReadBuffer(i,4);
  if i>0 then
    begin
      setlength(s,i);
      Stream.ReadBuffer(s[1],i);
      FRightBracket:=s;
    end;
  Stream.ReadBuffer(i,4);
  if i>0 then
    begin
      setlength(s,i);
      Stream.ReadBuffer(s[1],i);
      FWordName:=s;
    end;
  Stream.ReadBuffer(i,4);FPriority:=i;
  Stream.ReadBuffer(i,4);FItemClass:=TIAitemclass(i);
  Stream.ReadBuffer(i,4);FWholeWord:=TIAWholeWord(i);
  Stream.ReadBuffer(i,4);FMatchCase:=TIAMatchCase(i);

  if assigned(FonChange) then FOnChange(self);
end;

procedure TIAColorRichItem.SetFontColor(Value: TColor);
begin
  if FFontColor<>Value then
    begin
      FFontColor := Value;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItem.SetFontName(Value: TFontName);
begin
  if FFontName<>Value then
    begin
      FFontName := Value;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItem.SetFontSize(Value: integer);
begin
  if FFontSize<>Value then
    begin
      FFontSize := Value;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItem.SetFontStyle(Value: TFontStyles);
begin
  if Value<> FFontStyle then
    begin
      FFontStyle := Value;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItem.SetItemClass(Value: TIAitemclass);
begin
  if Value<> FItemClass then
    begin
      FItemClass := Value;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItem.SetLeftBracket(Value: string);
begin
  if Value<> FLeftBracket then
    begin
      FLeftBracket := Value;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItem.SetMatchCase(Value: TIAMatchCase);
begin
  if Value<> FMatchCase then
    begin
      FMatchCase := Value;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItem.SetPriority(Value: integer);
begin
  if Value<> FPriority then
    begin
      FPriority := Value;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItem.SetRangeBegin(Value: Byte);
begin
  if Value<> FRangeBegin then
    begin
      FRangeBegin := Value;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItem.SetRangeEnd(Value: Byte);
begin
  if Value<> FRangeEnd then
    begin
      FRangeEnd := Value;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItem.SetRightBracket(Value: string);
begin
  if Value<> FRightBracket then
    begin
      FRightBracket := Value;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItem.SetWholeWord(Value: TIAWholeWord);
begin
  if Value<> FWholeWord then
    begin
      FWholeWord := Value;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItem.SetWordName(Value: string);
begin
  if Value<> FWordName then
    begin
      FWordName := Value;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItem.WriteData(Stream: TStream);
  var i : cardinal;
begin
  Stream.WriteBuffer(FFontSize,4);
  Stream.WriteBuffer(FFontColor,4);
  i:=length(FFontName);
  Stream.WriteBuffer(i,4);
  if i>0 then Stream.WriteBuffer(FFontName[1],i);
  if fsBold in FFontStyle then
    begin
      i:=1;Stream.WriteBuffer(i,4);
    end else
    begin
      i:=0;Stream.WriteBuffer(i,4);
    end;
  if fsItalic in FFontStyle then
    begin
      i:=1;Stream.WriteBuffer(i,4);
    end else
    begin
      i:=0;Stream.WriteBuffer(i,4);
    end;
  if fsUnderline in FFontStyle then
    begin
      i:=1;Stream.WriteBuffer(i,4);
    end else
    begin
      i:=0;Stream.WriteBuffer(i,4);
    end;
  if fsStrikeOut in FFontStyle then
    begin
      i:=1;Stream.WriteBuffer(i,4);
    end else
    begin
      i:=0;Stream.WriteBuffer(i,4);
    end;    
  i:=FRangeEnd;
  Stream.WriteBuffer(i,4);
  i:=FRangeBegin;
  Stream.WriteBuffer(i,4);
  i:=length(FLeftBracket);
  Stream.WriteBuffer(i,4);
  if i>0 then Stream.WriteBuffer(FLeftBracket[1],i);
  i:=length(FRightBracket);
  Stream.WriteBuffer(i,4);
  if i>0 then Stream.WriteBuffer(FRightBracket[1],i);
  i:=length(FWordName);
  Stream.WriteBuffer(i,4);
  if i>0 then Stream.WriteBuffer(FWordName[1],i);
  Stream.WriteBuffer(FPriority,4);
  Stream.WriteBuffer(FItemClass,4);
  Stream.WriteBuffer(FWholeWord,4);
  Stream.WriteBuffer(FMatchCase,4);
end;

{ TIAColorRichItems }

procedure TIAColorRichItems.AddItem;
var TempItemsArray : TIAColorItemsArray;
    i : integer;
begin
  TempItemsArray:=Copy(FItemsArray,0,length(FItemsArray));
  SetLength(FItemsArray,length(FItemsArray)+1);
  FItemsArray[length(FItemsArray)-1]:=TIAColorRichItem.Create;
  FItemsArray[length(FItemsArray)-1].FonChange:=self.ItemsGhanged;
  for i:=0 to length(FItemsArray)-2 do FItemsArray[i]:=TempItemsArray[i];
  if assigned(FonChange) then FOnChange(self);
end;

procedure TIAColorRichItems.Assign(Source: TPersistent);
var i : cardinal;
begin
//  inherited Assign(Source);
  if Source is TIAColorRichItems then
    begin
      ItemsCount:=(Source as TIAColorRichItems).ItemsCount;
      for i:=1 to (Source as TIAColorRichItems).ItemsCount do self.Items[i-1].assign((Source as TIAColorRichItems).Items[i-1]);
    end;
end;

procedure TIAColorRichItems.ClearListofItems;
var i : cardinal;
begin
  for i:=0 to length(FItemsArray) do FItemsArray[i].free;
  setlength(FItemsArray,0);
end;

constructor TIAColorRichItems.Create;
begin
  inherited Create;
  setlength(FItemsArray,0);
  FOnChange:=nil;
end;

procedure TIAColorRichItems.DefineProperties(Filer: TFiler);
begin
  Filer.DefineBinaryProperty('Content',ReadData,WriteData,TRUE);
end;

procedure TIAColorRichItems.DeleteItem(Index: cardinal);
var TempItemsArray : TIAColorItemsArray;
    i : integer;
begin
  if Index<ItemsCount then
    begin
      setlength(TempItemsArray,length(FItemsArray));
      for i:=0 to length(FItemsArray)-1 do TempItemsArray[i]:=FItemsArray[i];
      FItemsArray[Index].free;
      setlength(FItemsarray,length(FItemsArray)-1);
      for i:=0 to integer(Index)-1 do           FItemsArray[i]:=TempItemsArray[i];
      for i:=Index to length(FItemsArray)-1 do    FItemsArray[i]:=TempItemsArray[i+1];
      setlength(TempItemsArray,0);
      if assigned(FonChange) then FOnChange(self);
    end;
end;

destructor TIAColorRichItems.Destroy;
var i : integer;
begin
  for i:=0 to length(FItemsArray)-1 do FItemsArray[i].Free;
  inherited Destroy;
end;

function TIAColorRichItems.GetItems(Index: Cardinal): TIAColorRichItem;
begin
  if Index<ItemsCount then  result:=FItemsArray[Index] else result:=nil;
end;

function TIAColorRichItems.GetItemsCount: Cardinal;
begin
  result:=length(FItemsArray);
end;

procedure TIAColorRichItems.ItemsGhanged(Sender: TObject);
begin
  if assigned(FonChange) then FOnChange(self);
end;

procedure TIAColorRichItems.ReadData(Stream: TStream);
var i,j : cardinal;
begin
  Stream.ReadBuffer(i,4);
  self.ItemsCount:=i;
  if i>0 then
    begin
      for j:=0 to i-1 do
        begin
          self.Items[j].readdata(Stream);
        end;
    end;
end;

procedure TIAColorRichItems.SetItems(Index: Cardinal;Value: TIAColorRichItem);
begin
  if Index<ItemsCount then
    begin
      FItemsArray[Index].assign(Value);
    end;
end;

procedure TIAColorRichItems.SetItemsCount(Value: Cardinal);
var i : cardinal;
begin
  if GetItemsCount<>Value then
    begin
      if  Value>GetItemsCount then
        begin
          for i:=1 to Value-GetItemsCount do self.AddItem;
        end else
        begin
          for i:=1 to GetItemsCount-Value do self.DeleteItem(length(FItemsArray)-1);
        end;
      if assigned(FonChange) then FOnChange(self);
    end;
end;

procedure TIAColorRichItems.WriteData(Stream: TStream);
var i : cardinal;
begin
  i:=self.ItemsCount;
  Stream.WriteBuffer(i,4);
  if self.ItemsCount>0 then
    begin
      for i:=0 to self.ItemsCount-1 do
        begin
          self.Items[i].writedata(Stream);
        end;
    end;
end;

end.
